// CDOWL : CD Audio Player for OWL


/******************************************************************************************

	CDOwl : Windows audio CD player using ObjectWindowsLibrary (OWL)

	Author : Phil Camp, 25-Mar-93
	CIS: 	100012,25 or 100042,3171

	Based on "CD" by Norbert Unterberg

 This is my first attempt a generating an OWL program. The main CD access classes are from
 the program "CD" by Norbert Unterburg. His original program was written in 'raw' Windows
 API calls rather than OWL. To gain experience in OWL I decided to produce a new version
 called "CDOwl".

 The program consists of the following files:

	CDOwl.CPP	- The main OWL program
	CDPlayer.CPP	- The CD player class that does all the hard work
        CDPlayer.H	- Include file for CDPlayer class
	MCI.CPP		- MCI access class that interfaces to Windows MCI
	MCI.H		- Include file for above
	CDOwl.H		- Resource constants include file
	CDOwl.RC	- The program's Windows resources


 The program seems to run under Windows 3.1 with no problems. You will need to copy
 the 'ini' file 'CDOWL.INI' to your home Windows directory (e.g. C:\WINDOWS) for the
 player to store and display CD titles.

 Most of the program has ported fairly well except for the CDPlayer class needing the
 program's main window handle. I'm not sure if there is a neater way to do this rather
 than simple passing it in the classe's constructor.

 ******************************************************************************************/

#include <owl.h>
#include <owlrc.h>
#include <window.h>
#include <string.h>
#include <mmsystem.h>
#include <toolhelp.h>
#include <stdio.h>
#include <string.h>


#include "mci.h"
#include "cdplayer.h"
#include "cdowl.h"



// ******************************* APPLICATION **********************************************

// Define a class derived from TApplication
class TCDApp :public TApplication
{
public:
  TCDApp(LPSTR AName, HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
    : TApplication(AName, hInstance, hPrevInstance, lpCmdLine, nCmdShow) {};
  virtual void InitMainWindow();
};


static CDPlayer* CD ;

// Define the main window as a dialog. This is based on the example code in the OWL manual.
class TCD : public TDialog
{
  public: TCD() ;
          virtual void WMInitDialog ( RTMessage Msg ) = [ WM_FIRST + WM_INITDIALOG ] ;
	  virtual void GetWindowClass(WNDCLASS&) ;
	  virtual LPSTR GetClassName() ;
          virtual void Timer    ( RTMessage Msg ) = [ WM_FIRST + WM_TIMER ] ;
	  virtual void BPlay    ( RTMessage Msg ) = [ ID_FIRST + ID_PLAY ] ;
	  virtual void BStop    ( RTMessage Msg ) = [ ID_FIRST + ID_STOP ] ;
	  virtual void BForward ( RTMessage Msg ) = [ ID_FIRST + ID_FORWARD ] ;
	  virtual void BBack    ( RTMessage Msg ) = [ ID_FIRST + ID_BACK ] ;
          virtual void BPause   ( RTMessage Msg ) = [ ID_FIRST + ID_PAUSE ];

          virtual void BQuit    ( RTMessage Msg ) = [ ID_FIRST + ID_QUIT ] ;
} ;

TCD::TCD() : TDialog (NULL, "CDOwl")
{} ;

void TCD::WMInitDialog ( RTMessage Msg)
{
  HWND hwnd ;
  hwnd = HWindow ;     // NB. Cannot do this in TCD constructor above as window not setup there
  // create and construct a new CDPlayer class
  CD = new CDPlayer(hwnd,GetApplication()->MainWindow);
  // open the audio CD channel
  MciError(hwnd, CD->Open("Player", MCI_OPEN_SHAREABLE)) ;
  // get the CDPlayer class into sync
  CD->Resync() ;

  TDialog::WMInitDialog(Msg) ;
}

LPSTR TCD::GetClassName()
{ // NB. must return the same name as the main dialog's resource name
  return "CDOwl" ;
}

void TCD::GetWindowClass( WNDCLASS& AWndClass)
{ WNDCLASS wc ;
  HINSTANCE Inst ;

  TDialog::GetWindowClass(AWndClass) ;
  // This code is to subclass bordlg so we display our own icon
  // NB. This is not 'normal' due to the borland custom dialog. This code was
  //     obtained from CIS and works.
  GetClassInfo(NULL, "bordlg", &wc);
  Inst = GetApplication()->hInstance ;
  wc.hIcon     = LoadIcon(Inst, MAKEINTRESOURCE(CDIcon));
  wc.hInstance = Inst;
  wc.lpszClassName = "BORDLG_1";
  RegisterClass(&wc);
}

void TCD::Timer ( RTMessage )	// pass Windows timer messages to the CDPlayer class
{
  CD->Timer() ;
}

// Dialog buttons

void TCD::BPlay ( RTMessage )
{
  CD->Play() ;
}

void TCD::BStop ( RTMessage )
{
  CD->Stop() ;
  CD->Resync() ;
}

void TCD::BForward ( RTMessage )
{
  CD->Forward() ;
  CD->Resync() ;
}

void TCD::BBack ( RTMessage )
{
  CD->Back() ;
  CD->Resync() ;
}

void TCD::BPause ( RTMessage )
{
  CD->Pause() ;
}

void TCD::BQuit ( RTMessage )
{
  // Close and delete the CDPlayer class
  if (CD)
    CD->Close();
  delete CD;

  //send a CLOSE message to the dialog
  SendMessage(HWindow,WM_SYSCOMMAND,SC_CLOSE,0L) ;
}

// **************************************** MAIN *********************************************


// Construct the TCDApp's MainWindow data member as a new TCD class
void TCDApp::InitMainWindow()
{
  MainWindow = new TCD();
}


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow)
{ HINSTANCE hBwcc;
  extern HINSTANCE hInst ;

  // windows setup
  hInst = hInstance;

  TCDApp CDApp ("CDApp", hInstance, hPrevInstance,
    lpCmdLine, nCmdShow);

  // ensure the borland custom controls DLL loaded
  SetErrorMode(SEM_NOOPENFILEERRORBOX);
  hBwcc = LoadLibrary("BWCC.DLL");
  if ((UINT)hBwcc < 32) {
     MessageBox(NULL, "Cannot load BWCC.DLL", "CDOWL",
                MB_ICONEXCLAMATION | MB_OK);
     return 0;
  }

  // run the application
  CDApp.Run();

  // free allocated resources

  FreeLibrary(hBwcc);

  return CDApp.Status;
}

