// CDPlayer class by Norbert Unterberg

#include <owl.h>
#include <owlrc.h>
#include <window.h>
#include <string.h>
#include <mmsystem.h>
#include <toolhelp.h>
#include <stdio.h>
#include <string.h>
#include <module.h>
#include <inputdia.h>

#include "mci.h"
#include "cdplayer.h"
#include "cdowl.h"


// ******************************** MISC *********************************************

HINSTANCE hInst;

char szAppName[] = "CD Player";

char *String(UINT id)
{
   static char szString[128];

   if (LoadString(hInst, id, szString, sizeof szString))
      return szString;
   else
      return "";
}

DWORD MciError(HWND hwnd, DWORD d)
{
   char s[256];

   if (d) {
      mciGetErrorString(d, s, sizeof s);
      MessageBox(hwnd, s, szAppName, MB_OK);
   }
    return d;
}

#define INI_NAME           "cdowl.ini"
#define INI_TOPIC_NAME     "Names"
#define INI_TOPIC_SETTINGS "Settings"
#define INI_POSITION       "Position"

// ******************************* CD *******************************************************
#define TIME_NONE          -999999

CDPlayer::CDPlayer(HWND aWindow, PTWindowsObject aMainWindow) : CDAudio(aWindow)
{
   nTracks = 0;
   LenTracks = NULL;
   mode = ModeNotReady;
   Track = 0;
   hwnd = aWindow;
   awnd = aMainWindow ;
   Sync = 0;
   dwPosition = 0;
   bCountUp = TRUE;
   wTimer = SetTimer(hwnd, 1, 1000, NULL);
}


CDPlayer::~CDPlayer()
{
   if (wTimer)
      KillTimer(hwnd, 1);
}


DWORD CDPlayer::Time()
{
   return GetTickCount();
/*
   TIMERINFO ti;

   ti.dwSize = sizeof ti;
   TimerCount(&ti);
   return ti.dwmsSinceStart;
*/
}


int CDPlayer::Synchronize(DWORD time)
{
   int t;
   BOOL bPlay;

   if (time == 0)
      time = Time();

   Sync = time + 5000;

   SetTimeFormat(MCI_FORMAT_MILLISECONDS);
   bPlay = Mode() == MCI_MODE_PLAY;

   if (mode == ModeNotReady) {
      dwPosition = 0;
      Track = 0;
   }

   if (mode == ModePlaying && !bPlay) {
      // CD has stopped
      dwPosition = PosTracks[1];
      Track = 0;
      mode = ModeStop;
   }
   if (bPlay)
      mode = ModePlaying;

   if (mode == ModePlaying) {
      dwPosition = Position();
      Track = CurrentTrack();
      StartTime = time - dwPosition;
      if (Sync > StartTime + PosTracks[Track] + LenTracks[Track])
         Sync = StartTime + PosTracks[Track] + LenTracks[Track];
   }
   return TRUE;
}


void CDPlayer::DisplayTime(LONG time, WORD wID)
{
   char buff[20];
   char buff2[20];

   if (time == TIME_NONE) {
      SetDlgItemText(hwnd, wID, " --:--");
      return;
   }

   if (time >= 0) {
      time /= 1000;
      wsprintf(buff, " %02d:%02d", int(time / 60), int(time % 60));
   } else {
      time = -(time / 1000) + 1;
      wsprintf(buff, "-%02d:%02d", int(time / 60), int(time % 60));
   }
   GetDlgItemText(hwnd, wID, buff2, sizeof buff2);
   if (strcmp(buff, buff2))
      SetDlgItemText(hwnd, wID, buff);
}


void CDPlayer::CheckReady()
{
   BOOL bReady = Ready();

   if (bReady && mode == ModeNotReady) {
      // CD-Laufwerk ist gerade Ready geworden
      HCURSOR hOldCur = SetCursor(LoadCursor(NULL, IDC_WAIT));
      mode = ModeScanning;
      UpdateDisplay(dwPosition);
      Login();
      SetCursor(hOldCur);
      mode = ModeStop;
      Sync = 0;
   }

   if (!bReady && mode != ModeNotReady ) {
      // Laufwerk wurde gerade "gestrt"
      Logout();
      mode = ModeNotReady;
      Sync = 0;
   }
}


int CDPlayer::Resync()
{
   CheckReady();
   Synchronize();
   UpdateDisplay(dwPosition);
   if (wTimer)
      KillTimer(hwnd, 1);
   wTimer = SetTimer(hwnd, 1, 1000, NULL);
   return TRUE;
}


void CDPlayer::UpdateDisplay(DWORD dwTime)
{
   static char buff[MAX_CDNAME];
   char *s;    // title string to display
   int t;      // track number to display
   static UINT modes[] = {
      IDS_NOTREADY,
      IDS_PAUSED,
      IDS_PLAYING,
      IDS_STOPPED,
      IDS_SCANNING
   };

   t = Track;
   s = *CDTitle ? CDTitle : String(modes[mode]);
   if (mode == ModeNotReady || mode == ModeScanning) {
      t = 0;
      s = String(modes[mode]);
      DisplayTime(TIME_NONE, ID_SINGLETIME);
      DisplayTime(TIME_NONE, ID_TOTALTIME);
   } else if (Track == 0) {
      t = nTracks;
      DisplayTime(0, ID_SINGLETIME);
      DisplayTime(PosTracks[nTracks+1], ID_TOTALTIME);
   } else {
      if (bCountUp) {
         DisplayTime(dwTime - PosTracks[Track], ID_SINGLETIME);
         DisplayTime(dwTime - PosTracks[1], ID_TOTALTIME);
      } else {
         DisplayTime(PosTracks[Track+1] - dwTime, ID_SINGLETIME);
         DisplayTime(PosTracks[nTracks+1] - dwTime, ID_TOTALTIME);
      }
   }

   if (GetDlgItemInt(hwnd, ID_TRACKCOUNT, NULL, FALSE) != t)
      SetDlgItemInt(hwnd, ID_TRACKCOUNT, t, FALSE);

   GetDlgItemText(hwnd, ID_TITLE, buff, sizeof(buff));
   if (strcmp(s, buff))
      SetDlgItemText(hwnd, ID_TITLE, s);
}


int CDPlayer::Timer()
{
   CheckReady();

   DWORD dwTime = Time();
   if (dwTime >= Sync) {
      // Synchronisation required
      Synchronize(dwTime);
   }

   if (mode == ModePlaying)
      dwPosition = dwTime - StartTime;
   UpdateDisplay(dwPosition);
   return TRUE;
}


void CDPlayer::SetTrack(int aTrack)
{
   Track = aTrack;
   SetPosition(PosTracks[Track]);
}


void CDPlayer::SetPosition(DWORD dwPos)
{
   dwPosition = dwPos;
   if (mode == ModePlaying) {
      SetTimeFormat(MCI_FORMAT_MILLISECONDS);
      MciError(hwnd, CDAudio::Play(MCI_FROM, dwPosition));
   }
   Resync();
}



int CDPlayer::Forward()
{
   int w = Track + 1;
   if (w > nTracks)
      return TRUE;
   SetTrack(w);
   return TRUE;
}


int CDPlayer::Back()
{
   int w = Track - 1;
   if (w < 1)
      return TRUE;
   SetTrack(w);
   return TRUE;
}


int CDPlayer::Pause()
{
   if (mode == ModePlaying) {
      SetTimeFormat(MCI_FORMAT_MILLISECONDS);
      dwPosition = Position();
      CDAudio::Stop();
   }
   mode = ModePause;
   Resync();
   return TRUE;
}


int CDPlayer::Play()
{
   if (mode == ModePlaying) {
      // if playing, play restarts the current song
      dwPosition = PosTracks[Track]; 
   }
   SetTimeFormat(MCI_FORMAT_MILLISECONDS);
   MciError(hwnd, CDAudio::Play(MCI_FROM, dwPosition));
   Resync();
   return TRUE;
}


int CDPlayer::Stop()
{
   CDAudio::Stop();
   mode = ModeStop;
   SetTrack(0);
   return TRUE;
}



int CDPlayer::Logout()
{
   EnableWindow(GetDlgItem(hwnd, ID_EDIT), FALSE);
   if (LenTracks) {
      delete LenTracks;
      delete PosTracks;
      LenTracks = NULL;
   }
   nTracks = 0;
   Track = 0;
   *CDTitle = '\0';
   return TRUE;
}


int CDPlayer::Login()
{
   int i;

   Logout();
   if (!Ready())
      return FALSE;

   nTracks = NumberOfTracks();
   LenTracks = new DWORD[nTracks+1];
   PosTracks = new DWORD[nTracks+2];

   SetTimeFormat(MCI_FORMAT_FRAMES);
   long n=Length();
   SetTimeFormat(MCI_FORMAT_MILLISECONDS);
   n=Length();

   SetTimeFormat(MCI_FORMAT_MILLISECONDS);
   for (i = 1; i <= nTracks; ++i) {
      PosTracks[i] = Position(i);
   }
   PosTracks[0] = PosTracks[1];

   LenTracks[0] = 0;
   for (i = 1; i < nTracks; ++i) {
      LenTracks[i] = PosTracks[i+1] - PosTracks[i];
   }
   LenTracks[nTracks] = Length(i);
   PosTracks[nTracks+1] = PosTracks[nTracks] + LenTracks[nTracks];
   dwPosition = PosTracks[1];
   Track = 0;

   wsprintf(CDID, "%08lx", n);
   if (!GetPrivateProfileString(INI_TOPIC_NAME, CDID,"",
 		   CDTitle, sizeof CDTitle, INI_NAME))
   { // unknown CD
     EditTitle() ;
   } ;
                                       
   if (!Ready()) {
      Logout();
      return FALSE;
   }
   EnableWindow(GetDlgItem(hwnd, ID_EDIT), TRUE);
   return TRUE;
}


void CDPlayer::EditTitle()
{
  strcpy(CDTitle,"") ;

  if ( awnd->GetModule()->ExecDialog(new TInputDialog(awnd,"CDOwl","Please enter CD title",CDTitle,sizeof(CDTitle))) == IDOK)
  {
    WritePrivateProfileString(INI_TOPIC_NAME,CDID,CDTitle,INI_NAME) ;
    Resync() ;
  } ;
}


void CDPlayer::ToggleUp()
{
   bCountUp ^= 1;
   Resync();
}

